/*
 * Copyright 2017, DornerWorks
 * Copyright 2017, Data61
 * Commonwealth Scientific and Industrial Research Organisation (CSIRO)
 * ABN 41 687 119 230.
 *
 * This software may be distributed and modified according to the terms of
 * the GNU General Public License version 2. Note that NO WARRANTY is provided.
 * See "LICENSE_GPLv2.txt" for details.
 *
 * @TAG(DATA61_DORNERWORKS_GPL)
 */
/*
 * This data was produced by DornerWorks, Ltd. of Grand Rapids, MI, USA under
 * a DARPA SBIR, Contract Number D16PC00107.
 *
 * Approved for Public Release, Distribution Unlimited.
 */

#ifndef __PLAT_MACHINE_H
#define __PLAT_MACHINE_H
#include <plat/machine/devices.h>
#include <machine/io.h>
#include <plat_mode/machine.h>

#define N_INTERRUPTS     96

#define MAX_IRQ (187)
enum IRQConstants {
//  INTERRUPT_RESERVED      =   0,
//  INTERRUPT_RESERVED      =   1,
//  INTERRUPT_RESERVED      =   2,
//  INTERRUPT_RESERVED      =   3,
//  INTERRUPT_RESERVED      =   4,
//  INTERRUPT_RESERVED      =   5,
//  INTERRUPT_RESERVED      =   6,
//  INTERRUPT_RESERVED      =   7,
//  INTERRUPT_RESERVED      =   8,
//  INTERRUPT_RESERVED      =   9,
//  INTERRUPT_RESERVED      =  10,
//  INTERRUPT_RESERVED      =  11,
//  INTERRUPT_RESERVED      =  12,
//  INTERRUPT_RESERVED      =  13,
//  INTERRUPT_RESERVED      =  14,
//  INTERRUPT_RESERVED      =  15,
//  INTERRUPT_RESERVED      =  16,
//  INTERRUPT_RESERVED      =  17,
//  INTERRUPT_RESERVED      =  18,
//  INTERRUPT_RESERVED      =  19,
//  INTERRUPT_RESERVED      =  20,
//  INTERRUPT_RESERVED      =  21,
//  INTERRUPT_RESERVED      =  22,
//  INTERRUPT_RESERVED      =  23,
//  INTERRUPT_RESERVED      =  24,
    INTERRUPT_CORE_VIRT_MAINT         = 25,
    INTERRUPT_CORE_HYP_TIMER          = 26,
    INTERRUPT_CORE_VIRT_TIMER         = 27,
    INTERRUPT_CORE_LEGACY_FIQ         = 28,
    INTERRUPT_CORE_SEC_PHYS_TIMER     = 29,
    INTERRUPT_CORE_NONSEC_PHYS_TIMER  = 30,
    INTERRUPT_CORE_LEGACY_IRQ         = 31,
//  INTERRUPT_RESERVED      =  25,
//  INTERRUPT_RESERVED      =  26,
//  INTERRUPT_RESERVED      =  27,
//  INTERRUPT_RESERVED      =  28,
//  INTERRUPT_RESERVED      =  29,
//  INTERRUPT_RESERVED      =  30,
//  INTERRUPT_RESERVED      =  31,
//  INTERRUPT_RESERVED      =  32,
//  INTERRUPT_RESERVED      =  33,
//  INTERRUPT_RESERVED      =  34,
//  INTERRUPT_RESERVED      =  35,
//  INTERRUPT_RESERVED      =  36,
//  INTERRUPT_RESERVED      =  37,
//  INTERRUPT_RESERVED      =  38,
//  INTERRUPT_RESERVED      =  39,


    INTERRUPT_RPU0_APM          = 40,
    INTERRUPT_RPU1_APM          = 41,
    INTERRUPT_OCM               = 42,
    INTERRUPT_LPD_APB           = 43,
    INTERRUPT_RPU0_ECC          = 44,
    INTERRUPT_RPU1_ECC          = 45,
    INTERRUPT_NAND              = 46,
    INTERRUPT_QSPI              = 47,
    INTERRUPT_GPIO              = 48,
    INTERRUPT_I2C0              = 49,
    INTERRUPT_I2C1              = 50,
    INTERRUPT_SPI0              = 51,
    INTERRUPT_SPI1              = 52,
    INTERRUPT_UART0             = 53,
    INTERRUPT_UART1             = 54,
    INTERRUPT_CAN0              = 55,
    INTERRUPT_CAN1              = 56,
    INTERRUPT_LPD_APM           = 57,
    INTERRUPT_RTC_ALARM         = 58,
    INTERRUPT_RTC_SECONDS       = 58,
    INTERRUPT_CLKMON            = 60,
    INTERRUPT_IPI_CH7           = 61,
    INTERRUPT_IPI_CH8           = 62,
    INTERRUPT_IPI_CH9           = 63,
    INTERRUPT_IPI_CH10          = 64,
    INTERRUPT_IPI_CH2           = 65,
    INTERRUPT_IPI_CH1           = 66,
    INTERRUPT_IPI_CH0           = 67,
    INTERRUPT_TTC0_0            = 68,
    INTERRUPT_TTC0_1            = 69,
    INTERRUPT_TTC0_2            = 70,
    INTERRUPT_TTC1_0            = 71,
    INTERRUPT_TTC1_1            = 72,
    INTERRUPT_TTC1_2            = 73,
    INTERRUPT_TTC2_0            = 74,
    INTERRUPT_TTC2_1            = 75,
    INTERRUPT_TTC2_2            = 76,
    INTERRUPT_TTC3_0            = 77,
    INTERRUPT_TTC3_1            = 78,
    INTERRUPT_TTC3_2            = 79,
    INTERRUPT_SDIO0             = 80,
    INTERRUPT_SDIO1             = 81,
    INTERRUPT_SDIO0_WAKEUP      = 82,
    INTERRUPT_SDIO1_WAKEUP      = 83,
    INTERRUPT_LPD_SWDT          = 84,
    INTERRUPT_CSU_SWDT          = 85,
    INTERRUPT_LPD_ATB           = 86,
    INTERRUPT_AIB               = 87,
    INTERRUPT_SYSMON            = 88,
    INTERRUPT_GEM0              = 89,
    INTERRUPT_GEM0_WAKEUP       = 90,
    INTERRUPT_GEM1              = 91,
    INTERRUPT_GEM1_WAKEUP       = 92,
    INTERRUPT_GEM2              = 93,
    INTERRUPT_GEM2_WAKEUP       = 94,
    INTERRUPT_GEM3              = 95,
    INTERRUPT_GEM3_WAKEUP       = 96,
    INTERRUPT_USB0_ENDPOINT_0   = 97,
    INTERRUPT_USB0_ENDPOINT_1   = 98,
    INTERRUPT_USB0_ENDPOINT_2   = 99,
    INTERRUPT_USB0_ENDPOINT_3   = 100,
    INTERRUPT_USB0_OTG          = 101,
    INTERRUPT_USB1_ENDPOINT_0   = 102,
    INTERRUPT_USB1_ENDPOINT_1   = 103,
    INTERRUPT_USB1_ENDPOINT_2   = 104,
    INTERRUPT_USB1_ENDPOINT_3   = 105,
    INTERRUPT_USB1_OTG          = 106,
    INTERRUPT_USB0_WAKEUP       = 107,
    INTERRUPT_USB1_WAKEUP       = 108,
    INTERRUPT_LPD_DMA_0         = 109,
    INTERRUPT_LPD_DMA_1         = 110,
    INTERRUPT_LPD_DMA_2         = 111,
    INTERRUPT_LPD_DMA_3         = 112,
    INTERRUPT_LPD_DMA_4         = 113,
    INTERRUPT_LPD_DMA_5         = 114,
    INTERRUPT_LPD_DMA_6         = 115,
    INTERRUPT_LPD_DMA_7         = 116,
    INTERRUPT_CSU               = 117,
    INTERRUPT_CSU_DMA           = 118,
    INTERRUPT_EFUSE             = 119,
    INTERRUPT_XPPU              = 120,
    INTERRUPT_PL_PS_GROUP0_0    = 121,
    INTERRUPT_PL_PS_GROUP0_1    = 122,
    INTERRUPT_PL_PS_GROUP0_2    = 123,
    INTERRUPT_PL_PS_GROUP0_3    = 124,
    INTERRUPT_PL_PS_GROUP0_4    = 125,
    INTERRUPT_PL_PS_GROUP0_5    = 126,
    INTERRUPT_PL_PS_GROUP0_6    = 127,
    INTERRUPT_PL_PS_GROUP0_7    = 128,
//  INTERRUPT_RESERVED_0        = 129,
//  INTERRUPT_RESERVED_1        = 130,
//  INTERRUPT_RESERVED_2        = 131,
//  INTERRUPT_RESERVED_3        = 132,
//  INTERRUPT_RESERVED_4        = 133,
//  INTERRUPT_RESERVED_5        = 134,
//  INTERRUPT_RESERVED_6        = 135,
    INTERRUPT_PL_PS_GROUP1_0    = 136,
    INTERRUPT_PL_PS_GROUP1_1    = 137,
    INTERRUPT_PL_PS_GROUP1_2    = 138,
    INTERRUPT_PL_PS_GROUP1_3    = 139,
    INTERRUPT_PL_PS_GROUP1_4    = 140,
    INTERRUPT_PL_PS_GROUP1_5    = 141,
    INTERRUPT_PL_PS_GROUP1_6    = 142,
    INTERRUPT_PL_PS_GROUP1_7    = 143,
    INTERRUPT_DDR               = 144,
    INTERRUPT_FPD_SWDT          = 145,
    INTERRUPT_PCIE_MSI0         = 146,
    INTERRUPT_PCIE_MSI1         = 147,
    INTERRUPT_PCIE_INTX         = 148,
    INTERRUPT_PCIE_DMA          = 149,
    INTERRUPT_PCIE_MSC          = 150,
    INTERRUPT_DISPLAYPORT       = 151,
    INTERRUPT_FPD_APB           = 152,
    INTERRUPT_FPD_ATB           = 153,
    INTERRUPT_DPDMA             = 154,
    INTERRUPT_FPD_ATM           = 155,
    INTERRUPT_FPD_DMA_0         = 156,
    INTERRUPT_FPD_DMA_1         = 157,
    INTERRUPT_FPD_DMA_2         = 158,
    INTERRUPT_FPD_DMA_3         = 159,
    INTERRUPT_FPD_DMA_4         = 160,
    INTERRUPT_FPD_DMA_5         = 161,
    INTERRUPT_FPD_DMA_6         = 162,
    INTERRUPT_FPD_DMA_7         = 163,
    INTERRUPT_GPU               = 164,
    INTERRUPT_SATA              = 165,
    INTERRUPT_FPD_XMPU          = 166,
    INTERRUPT_APU_VCPUMNT_0     = 167,
    INTERRUPT_APU_VCPUMNT_1     = 168,
    INTERRUPT_APU_VCPUMNT_2     = 169,
    INTERRUPT_APU_VCPUMNT_3     = 170,
    INTERRUPT_CPU_CTI_0         = 171,
    INTERRUPT_CPU_CTI_1         = 172,
    INTERRUPT_CPU_CTI_2         = 173,
    INTERRUPT_CPU_CTI_3         = 174,
    INTERRUPT_PMU_COMM_0        = 175,
    INTERRUPT_PMU_COMM_1        = 176,
    INTERRUPT_PMU_COMM_2        = 177,
    INTERRUPT_PMU_COMM_3        = 178,
    INTERRUPT_APU_COMM_0        = 179,
    INTERRUPT_APU_COMM_1        = 180,
    INTERRUPT_APU_COMM_2        = 181,
    INTERRUPT_APU_COMM_3        = 182,
    INTERRUPT_L2_CACHE          = 183,
    INTERRUPT_APU_EXTERROR      = 184,
    INTERRUPT_APU_REGERROR      = 185,
    INTERRUPT_CCI               = 186,
    INTERRUPT_SMMU              = 187,
    maxIRQ = MAX_IRQ
} platform_interrupt_t;

#define KERNEL_TIMER_IRQ INTERRUPT_CORE_VIRT_TIMER
#define KERNEL_PMU_IRQ   INTERRUPT_PMU_COMM_0

#include <arch/machine/gic_pl390.h>

#endif /* !__PLAT_MACHINE_H */
